UNIT Shared;

INTERFACE

USES
  Windows, SysUtils;

CONST
  TXT_APP    = 'ImageJ';
  CFG_EXT    = '.cfg';

  JAR_IMAGEJ = 'ij.jar';
  JAR_TOOLS  = 'lib\tools.jar';

  EXE_JVM    = 'javaw.exe';

  DEF_JVM    = 'bin\'+EXE_JVM;
  DEF_JRE    = 'jre\'+DEF_JVM; // ImageJ installed
  DEF_CFG    = TXT_APP+CFG_EXT;
  DEF_PARAMS = '-Xmx%dm -cp %s'+JAR_IMAGEJ+' ij.ImageJ';

  NFG_JVM1   = '1.4.2'; // Exclude this if found

  REG_JRE    = '\SOFTWARE\JavaSoft\Java Runtime Environment';
  REG_JDK    = '\SOFTWARE\JavaSoft\Java Development Kit';
  REG_CUR    = 'CurrentVersion';
  REG_JHM    = 'JavaHome';

// Program functions
procedure FatalError(sMsg,sTitle: String; nErrCode: Integer);
function IJJarExists(sIJJar: String): Boolean;
function GetValidIJPath(var sIJPath: String; bDotFmt: Boolean): Boolean;
function GetValidJVM(sIJPath: String; var sJVMFile: String): Boolean;
function CreateCfgFile(sDefCfg,sDefDir,sDefJVM,sDefParams: String;
  bPrompt: Boolean = false): String;
function ReadCfgFile(sCfgFile: String; var sGetDir,sGetJVM,sGetParams: String): Boolean;
function PassedParams: String;

// General utility functions
function EvalSet(bEval: Boolean; var bResult: Boolean): Boolean; overload;
function EvalSet(sEval: String; var sResult: String): String; overload;
function EvalSet(nEval: Integer; var nResult: Integer): Integer; overload;
function GetShortPathString(sLongPathName: String): String;
function GetSpecialFolderPath(nFolder: Integer; bTrailingSlash: Boolean = true): String;
function NormalizeDir(sPath: String; bTrailingSlash: Boolean = true): String;

IMPLEMENTATION

uses
  ActiveX, ShlObj, Registry, OpenDlg;

// -----------------------------------------------------------------------

// Display error message and exit with error code
procedure FatalError(sMsg,sTitle: String; nErrCode: Integer);
begin
  MessageBox(0,PChar(sMsg),PChar(sTitle),MB_OK or MB_ICONERROR);
  halt(nErrCode);
end;

function IJJarExists(sIJJar: String): Boolean;
begin
  if not(EvalSet(FileExists(sIJJar),Result)) then
    Result:=FileExists(NormalizeDir(sIJJar)+JAR_IMAGEJ);
end;

function GetValidIJPath(var sIJPath: String; bDotFmt: Boolean): Boolean;
begin
  if (EvalSet(Trim(sIJPath),sIJPath) = '') or (sIJPath = '.') then
    sIJPath:=ExtractFilePath(ParamStr(0));
  sIJPath:=NormalizeDir(sIJPath)+JAR_IMAGEJ;
  // If no file, then try default install folder...
  if not(FileExists(sIJPath)) then
    begin
      sIJPath:=GetSpecialFolderPath(CSIDL_PROGRAM_FILES)+TXT_APP+'\'+JAR_IMAGEJ;
      // If no file, then setup default search location
      if not(FileExists(sIJPath)) then
        sIJPath:=GetSpecialFolderPath(CSIDL_PROGRAM_FILES)+JAR_IMAGEJ;
    end;
  if FileExists(sIJPath) or
     OpenDialog(0,'Select the '+TXT_APP+' JAR File',ExtractFilePath(sIJPath),
                TXT_APP+' ('+JAR_IMAGEJ+')|'+JAR_IMAGEJ+'|All Files (*.*)|*.*',sIJPath) then
    begin
      sIJPath:=ExtractFilePath(sIJPath);
      if bDotFmt and SameText(sIJPath,ExtractFilePath(ParamStr(0))) then
        sIJPath:='.';
      Result:=true;
    end
  else
    Result:=false;
end;

function GetValidJVM(sIJPath: String; var sJVMFile: String): Boolean;
var
  reg: TRegistry;
  bJRE,bJDK: Boolean;
  sJRE,sJDK: String;

function GetJPath(sRegPath: String; var sPath: String): Boolean;
var
  sVersion: String;
begin
  Result:=false;
  if reg.OpenKey(REG_JRE,false) then
    try
      sPath:=reg.ReadString(REG_CUR);
      reg.CloseKey;
      if reg.OpenKey(NormalizeDir(sRegPath)+sPath,false) then
        begin
          sPath:=NormalizeDir(reg.ReadString(REG_JHM))+DEF_JVM;
          // Exclude NFG_JVM1 here (list could be longer)
          Result:=(Pos(NFG_JVM1,sPath) = 0) and FileExists(sPath);
          reg.CloseKey;
        end;
    except
      reg.CloseKey;
    end;
end;

begin
  Result:=false;
  // Check for the default jre installed by ImageJ
  if (sIJPath = '') or (sIJPath = '.') then
    sIJPath:=ExtractFilePath(ParamStr(0));
  sIJPath:=NormalizeDir(sIJPath);
  if FileExists(sIJPath+DEF_JRE) then
    begin
      sJVMFile:=sIJPath+DEF_JRE;
      Result:=true;
    end
  else // Check the registry
    try
      reg:=TRegistry.Create;
      reg.RootKey:=HKEY_LOCAL_MACHINE;
      // Get the JRE location
      bJRE:=GetJPath(REG_JRE,sJRE);
      bJDK:=GetJPath(REG_JDK,sJDK);
      if bJRE then
        begin
          if bJDK then // Have BOTH - ask user
            begin
              if (MessageBox(0,PChar('Both the JDK and the JRE have been detected '+
                  'on your system as:'+#13#13+'JDK = '+sJDK+#13+'JRE = '+sJRE+#13#13+
                  'Would you like to use the JDK virtual machine (otherwise the JRE '+
                  'will be selected)?'),'Java Virtual Machine',
                   MB_YESNO or MB_ICONQUESTION) = ID_YES) then
                begin
                  sJVMFile:=sJDK;
                  Result:=true;
                end;
            end;
          if not(Result) then
            begin
              sJVMFile:=sJRE;
              Result:=true;
            end;
        end
      else
        if bJDK then
          begin
            sJVMFile:=sJDK;
            Result:=true;
          end;
    finally
      reg.Free;
    end;
  // Could not find JVM in registry - ask user
  if not(Result) then
    begin
      sJVMFile:=EXE_JVM;
      Result:=OpenDialog(0,'Select the JVM for '+TXT_APP,
                         GetSpecialFolderPath(CSIDL_PROGRAM_FILES),
                         'JVM ('+EXE_JVM+')|'+EXE_JVM+'|All Files (*.*)|*.*',sJVMFile);
    end;
end;

function CreateCfgFile(sDefCfg,sDefDir,sDefJVM,sDefParams: String;
  bPrompt: Boolean = false): String;
var
  fCfg: Text;
  bCleanup: Boolean;
  mst: TMemoryStatus;
  sJTools: String;
begin
  bCleanup:=false;
  if (Trim(sDefCfg) = '') then
    Result:=ChangeFileExt(ParamStr(0),CFG_EXT)
  else
    Result:=sDefCfg;
  if bPrompt then
    MessageBox(0,'The program will now be auto-configured. You may be prompted '+
               'to input the locations of various files that can not be auto-detected. '+
               'Please consult the installation instructions for further assistance.',
               PChar('Welcome to '+TXT_APP),MB_OK or MB_ICONINFORMATION);
  try
    assignFile(fCfg,Result);
    rewrite(fCfg);
    bCleanup:=true;
    // Line 1
    if GetValidIJPath(sDefDir,true) then
      writeln(fCfg,sDefDir)
    else
      Abort;
    // Line 2
    if GetValidJVM(sDefDir,sDefJVM) then
      writeln(fCfg,sDefJVM)
    else
      Abort;
    // Line 3
    if (Trim(sDefParams) = '') then
      begin
        mst.dwLength:=SizeOf(mst);
        GlobalMemoryStatus(mst);
        // Check for Tools.jar
        sDefJVM:=ExtractFilePath(sDefJVM); // Get folder
        Delete(sDefJVM,Length(sDefJVM),1); // Remove trailing '\'
        sJTools:=ExtractFilePath(sDefJVM)+JAR_TOOLS;
        if FileExists(sJTools) then
          sJTools:=GetShortPathString(sJTools)+';'
        else
          sJTools:='';
        // Default to 2/3 available physical memory in MB
        sDefParams:=Format(DEF_PARAMS,[((mst.dwTotalPhys div $300000)*2),sJTools]);
      end;
    writeln(fCfg,sDefParams);
    closeFile(fCfg);
  except
    if bCleanup then
      begin
        closeFile(fCfg);
        DeleteFile(Result); // Cleanup
      end;
    FatalError('Unable to create the configuration file:'+#13#13+Result+#13#13+
               'Please check the installation folder and try again.',
               'Configuration File Error',1);
  end;
end;

function ReadCfgFile(sCfgFile: String; var sGetDir,sGetJVM,sGetParams: String): Boolean;
var
  fCfg: Text;
  bCleanup: Boolean;
begin
  Result:=false;
  bCleanup:=false;
  // Read the config file for info - load each string then trim whitespace chars
  try
    assignFile(fCfg,sCfgFile);
    reset(fCfg);
    bCleanup:=true;
    readln(fCfg,sGetDir);
    sGetDir:=Trim(sGetDir);
    if (sGetDir = '') or (sGetDir = '.') then
      sGetDir:=ExtractFilePath(sCfgFile);
    readln(fCfg,sGetJVM);
    sGetJVM:=Trim(sGetJVM);
    readln(fCfg,sGetParams);
    sGetParams:=Trim(sGetParams);
    if (sGetParams = '') then // Missing JVM or Parameters?
      Abort;
    closeFile(fCfg);
    Result:=true; // All OK
  except
    if bCleanup then
      closeFile(fCfg);
  end;
end;

// Parse command line for passed files
function PassedParams: String;
var
  nPos: Integer;
begin
  Result:=GetCommandLine;
  if (EvalSet(Pos(UpperCase(ParamStr(0)),UpperCase(Result)),nPos) > 0) then
    Result:=Copy(Result,nPos+Length(ParamStr(0))+1,Length(Result));
  Result:=Trim(Result);
end;

// -----------------------------------------------------------------------

function EvalSet(bEval: Boolean; var bResult: Boolean): Boolean;
begin
  bResult:=bEval;
  Result:=bEval;
end;

function EvalSet(sEval: String; var sResult: String): String;
begin
  sResult:=sEval;
  Result:=sEval;
end;

function EvalSet(nEval: Integer; var nResult: Integer): Integer;
begin
  nResult:=nEval;
  Result:=nEval;
end;

function GetShortPathString(sLongPathName: String): String;
var
  nLen: Integer;
begin
  Result:=sLongPathName;
  if (EvalSet(GetShortPathName(PChar(sLongPathName),nil,0),nLen) > 0) then
    begin
      SetLength(Result,nLen-1);
      GetShortPathName(PChar(sLongPathName),PChar(Result),nLen);
    end;
end;

function FreePIDL(var pidl: PItemIdList): Boolean;
var
  pMalloc: IMalloc;
begin
  pMalloc:=nil;
  if Assigned(pidl) then
    begin
      // Not sure if DidAlloc code is really necessary?  2004-02-01
      if Succeeded(SHGetMalloc(pMalloc)) then  // and (pMalloc.DidAlloc(pidl) > 0) then
        begin
          pMalloc.Free(pidl);
          pidl:=nil;
          Result:=true;
        end
      else
        Result:=false;
    end
  else
    Result:=true;
end;

function GetSpecialFolderPath(nFolder: Integer; bTrailingSlash: Boolean = true): String;
var
  pidl: PItemIDList;
  szPath: Array[0..MAX_PATH] of Char;
begin
  Result:='';
  pidl:=nil;
  FillChar(szPath,SizeOf(szPath),#0);
  if Succeeded(SHGetSpecialFolderLocation(0,nFolder,pidl)) then
    begin
      SHGetPathFromIDList(pidl,szPath);
      FreePIDL(pidl);
    end;
  Result:=Trim(szPath);
  if bTrailingSlash and (Result <> '') and (Result[Length(Result)] <> '\') then
    Result:=Result+'\';
end;

function NormalizeDir(sPath: String; bTrailingSlash: Boolean = true): String;
var
  sRoot: String;
  nPos: Integer;
begin
  Result:=Trim(sPath);
  if (Result <> '') then
    begin
      while (Pos('/',Result) <> 0) do
        Result[Pos('/',Result)]:='\';
      // This prevents path from ending in multi-"\"
      while (Result <> '') and (Result[Length(Result)] = '\') do
        Delete(Result,Length(Result),1);
      if bTrailingSlash then
        Result:=Result+'\';
      while (EvalSet(Pos('\..',Result),nPos) <> 0) and
            (Pos('\',EvalSet(Copy(Result,1,nPos-1),sRoot)) <> 0) do
        begin
          while (sRoot <> '') and (sRoot[Length(sRoot)] <> '\') do
            Delete(sRoot,Length(sRoot),1);
          Result:=sRoot+Copy(Result,nPos+4,Length(Result)-nPos); // 4 = Length('\..')+1
        end;
    end;
end;

INITIALIZATION
  CoInitialize(nil);
FINALIZATION
  CoUninitialize;
END.
